--[[
Copyright (c) 2008-2009, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true );

if( not TR ) then
    error( ("%s requires %s"):format( L["Calendar Broker"], "LibLordFarlander-TableRecycler-1.0" ) );
end--if

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Calendar_Broker" );

-- Constants
CALENDARBROKER_SECPERMIN = 60;
CALENDARBROKER_SECPERHOUR = 60 * CALENDARBROKER_SECPERMIN;
CALENDARBROKER_SECPERDAY = 24 * CALENDARBROKER_SECPERHOUR;

CalendarBroker.Calendars = {};
CalendarBroker.events = {};
CalendarBroker.Initialized = false;

function CalendarBroker:PLAYER_ENTERING_WORLD()
    if( not self.Initialized ) then
        for _, handler in pairs( self.Calendars ) do
            if( handler.PLAYER_ENTERING_WORLD ) then
                handler:PLAYER_ENTERING_WORLD( self );
            end--if
        end--for
        self:CalendarUpdate( "ALL" );
        self.Initialized = true;
        self:UnregisterEvent( "PLAYER_ENTERING_WORLD" );
    end--if
end--CalendarBroker:PLAYER_ENTERING_WORLD()

function CalendarBroker:SetupCalendarSupport()
    for _, handler in pairs( self.Calendars ) do
        if( handler.SetupCalendarSupport ) then
            handler:SetupCalendarSupport( self );
        end--if
    end--for
    self:RegisterEvent( "PLAYER_ENTERING_WORLD" );
end--CalendarBroker:SetupCalendarSupport()

function CalendarBroker.SecondsFromEpoch( month, day, year, hour, min, sec )
    if( type( month ) == "table" ) then
        return time( month );
    else
        local tim = TR:GetTable( "year", year, "month", month, "day", day, "hour", hour or 0, "min", min or 0, "sec", sec or 0 );
        local secs = time( tim );

        TR:ReturnTable( tim );
        return secs;
    end--if
end--CalendarBroker.SecondsFromEpoch( month, day, year, hour, min, sec )

function CalendarBroker.PrintDate( seconds )
    -- Used to debug the math
    return date( "%m/%d/%y %H:%M:%S", seconds );
end--CalendarBroker.PrintDate( seconds )

function CalendarBroker:PrintDebug( text )
    if( self.db.profile.DebugMessages ) then
        self:Print( "|cffffff00DEBUG |cffffffff" .. text )
    end--if
end--CalendarBroker:PrintDebug( text )

function CalendarBroker:GetCalendarDefaults()
    local returnVal = {};
    
    for caltype, handler in pairs( self.Calendars ) do
        if( handler.CanEnableCalendar and handler:CanEnableCalendar() or ( not handler.CanEnableCalendar ) ) then
            returnVal[caltype] = true;
        end--if
    end--for
    return returnVal;
end--CalendarBroker:GetCalendarDefaults()

function CalendarBroker:GetCalendarOptionsGroup()
    local returnVal = {
        type = "group",
        handler = self,
        name = L["Calendars"],
        args = {},
    };
    
    for caltype, handler in pairs( self.Calendars ) do
        if( handler.CanEnableCalendar and handler:CanEnableCalendar() or ( not handler.CanEnableCalendar ) ) then
            returnVal.args[caltype:gsub( "([%c%s%p])", function( input ) return ("_%03i"):format( strbyte( input ) ) end )] = {
                type = "toggle",
                name = L["Use %s calendar"]:format( caltype ),
                desc = L["Use %s calendar"]:format( caltype ),
                width = "double",
                get = function( info )
                          return info.handler.db.profile.Calendars[caltype];
                      end,
                set = function( info, value )
                          info.handler.db.profile.Calendars[caltype] = value;
                          if( info.handler.Calendars[caltype].SetupCalendarSupport ) then
                              info.handler.Calendars[caltype]:SetupCalendarSupport( info.handler );
                          end--if
                          info.handler:CalendarUpdate( "ALL" );
                      end,
            };
        end--if
    end--for
    return returnVal;
end--CalendarBroker:GetCalendarOptionsGroup()

function CalendarBroker:RemoveAllEvents( calendarType )
    for index, event in pairs( self.events ) do
        if( event.source == calendarType ) then
            TR:ReturnTable( table.remove( self.events, index ) );
        end--if
    end--for
end--CalendarBroker:RemoveAllEvents

function CalendarBroker:CalendarUpdate( calendarType )
    self:GetCalendarEvents( calendarType );
    self:Update();
end--CalendarBroker:CalendarUpdate( calendarType );

function CalendarBroker:GetCalendarEvents( calendarType )
    local EnabledCalendars = self.db.profile.Calendars;
    local Calendars = self.Calendars;
    local actioned = false;

    if( not calendarType ) then
        calendarType = "ALL";
    end--if
--     if( self.events and ( calendarType ~= "ALL" ) ) then
--         self:RemoveAllEvents( calendarType );
--     elseif( self.events ) then
--         TR:ReturnTable( self.events );
--     else
--         calendarType = "ALL";
--     end--if
    if( not self.events ) then
        self.events = TR:GetTable();
    end--if

    if( calendarType == "ALL" ) then
        for caltype, handler in pairs( self.Calendars ) do
            if( EnabledCalendars[caltype] and  handler.GetCalendar ) then
                handler:GetCalendar( self, self.db.profile.MaxNumberOfDays );
                actioned = true;
            end--if
        end--for
    elseif( EnabledCalendars[calendarType] ) then
        local handler = Calendars[calendarType];

        if( handler and handler.GetCalendar ) then
            handler:GetCalendar( self, self.db.profile.MaxNumberOfDays );
            actioned = true;
        end--if
    end--if
end--CalendarBroker:GetCalendarEvents( calendarType )

function CalendarBroker:AddEvent( start, duration, text, eventType, calendar, eventID )
    -- start and duration are in units of seconds
    local EndTime = start + duration;
    local curTime = time();
    local MaxStartTime = curTime + ( self.db.profile.MaxNumberOfDays + 1 ) * CALENDARBROKER_SECPERDAY;

    if( not eventType ) then
        eventType = "NONE";
    end--if
    if( not calendar ) then
        calendar = "NONE";
    end--if
    if( ( EndTime >= curTime ) and ( start < MaxStartTime ) ) then
        local events = self.events;
        
        for index, event in ipairs( events ) do
            if( ( event.source == calendar ) and ( event.id == eventID ) ) then
                --duplicate event
                if( event.startTime > start ) then
                    event.startTime = start;
                end--if
                if( event.endTime < EndTime ) then
                    event.endTime = EndTime;
                end--if
                event.length = event.endTime - event.startTime;
                return;
            elseif( event.startTime > start ) then
                table.insert( events, index, TR:GetTable( "startTime", start, "title", text, "type", eventType, "endTime", EndTime, "source", calendar, "length", duration, "id", eventID ) );
                return;
            end--if
        end--if
        table.insert( events, TR:GetTable( "startTime", start, "title", text, "type", eventType, "endTime", EndTime, "source", calendar, "length", duration, "id", eventID ) );
    end--if
end--CalendarBroker:AddEvent( start, duration, text )
